local floor,ceil = math.floor,math.ceil

local function getChunkImage(self,x,y)
	local row = self[x]
	if (row==nil) then
		row = {}
		self[x] = row
	end
	local image = row[y]
	if (image==nil) then
		local chunkImageData = self.chunkImageData
		local chunkSize = self.chunkSize
		chunkImageData:paste(self.imageData,0,0,x*chunkSize,y*chunkSize,chunkSize,chunkSize)
		image = love.graphics.newImage(chunkImageData)
		row[y] = image
	end
	return image
end

local function tiledImage_draw(self,x,y,width,height) --input drawn world coordinates
	local chunkSize = self.chunkSize
	local chunkWidth,chunkHeight = chunkSize,chunkSize
	local chunkCount_x,chunkCount_y = self.chunkCount_x,self.chunkCount_y
	local x1,y1 = floor(x/chunkWidth),floor(y/chunkHeight)
	local x2,y2 = floor((x+width)/chunkWidth),floor((y+height)/chunkHeight)
	for x=x1,x2 do
		for y=y1,y2 do
			local cx,cy = x%chunkCount_x,y%chunkCount_y
			local px,py = x*chunkWidth,y*chunkHeight
			love.graphics.draw(getChunkImage(self,cx,cy),px,py)
		end
	end
end

local function newTiledImage(imageData,chunkSize)
	local width,height = imageData:getDimensions()
	local chunkCount_x = ceil(width/chunkSize)
	local chunkCount_y = ceil(height/chunkSize)
	
	local chunkImageData = love.image.newImageData(chunkSize,chunkSize)
	
	local object = {
		chunkSize = chunkSize,
		chunkCount_x = chunkCount_x,
		chunkCount_y = chunkCount_y,
		
		imageData = imageData,
		chunkImageData = chunkImageData,
		
		draw = tiledImage_draw,
	}
	
	return object
end

return newTiledImage